﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows;
using System.Windows.Interop;
using System.Windows.Media;
using System.Windows.Media.Imaging;

public static class ProcessHelper
{
    private static readonly Dictionary<string, ImageSource> _smallIconCache = new Dictionary<string, ImageSource>();
    private static readonly Dictionary<string, ImageSource> _largeIconCache = new Dictionary<string, ImageSource>();

    public static ImageSource FindIcon(string fileName, bool large)
    {
        var extension = Path.GetExtension(fileName);
        if (extension == null)
            return null;
        var cache = large ? _largeIconCache : _smallIconCache;
        ImageSource icon;
        if (cache.TryGetValue(extension, out icon))
            return icon;

        icon = Imaging.CreateBitmapSourceFromHIcon(GetFileIcon(fileName, large ? IconSize.Large : IconSize.Small, false).Handle, Int32Rect.Empty, BitmapSizeOptions.FromEmptyOptions());
        cache.Add(extension, icon);
        return icon;
    }

    public static string FindExecutable(string pathToFile)
    {
        StringBuilder objResultBuffer = new StringBuilder(1024);
        long lngResult = Shell32.FindExecutable(pathToFile, string.Empty, objResultBuffer);

        if (lngResult >= 32)
            return objResultBuffer.ToString();

        return string.Format(
            "Error: ({0})", lngResult);
    }

    private enum IconSize
    {
        Large = 0, //32
        Small = 1  //16
    }

    private static Icon GetFileIcon(string name, IconSize size, bool linkOverlay)
    {
        var shfi = new Shell32.Shfileinfo();
        var flags = Shell32.ShgfiIcon | Shell32.ShgfiUsefileattributes;
        if (linkOverlay) flags += Shell32.ShgfiLinkoverlay;

        flags += size == IconSize.Small ? Shell32.ShgfiSmallicon : Shell32.ShgfiLargeicon;

        Shell32.SHGetFileInfo(name,
            Shell32.FileAttributeNormal,
            ref shfi,
            (uint)Marshal.SizeOf(shfi),
            flags);

        var icon = (Icon)Icon.FromHandle(shfi.hIcon).Clone();
        Shell32.DestroyIcon(shfi.hIcon);     // Cleanup
        return icon;
    }

    static class Shell32
    {
        private const int MaxPath = 256;
        [StructLayout(LayoutKind.Sequential)]
        public struct Shfileinfo
        {
            private const int Namesize = 80;
            public readonly IntPtr hIcon;
            private readonly int iIcon;
            private readonly uint dwAttributes;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = MaxPath)]
            private readonly string szDisplayName;
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = Namesize)]
            private readonly string szTypeName;
        };
        public const uint ShgfiIcon = 0x000000100;     // get icon
        public const uint ShgfiLinkoverlay = 0x000008000;     // put a link overlay on icon
        public const uint ShgfiLargeicon = 0x000000000;     // get large icon
        public const uint ShgfiSmallicon = 0x000000001;     // get small icon
        public const uint ShgfiUsefileattributes = 0x000000010;     // use passed dwFileAttribute
        public const uint FileAttributeNormal = 0x00000080;
        [DllImport("Shell32.dll")]
        public static extern IntPtr SHGetFileInfo(
            string pszPath,
            uint dwFileAttributes,
            ref Shfileinfo psfi,
            uint cbFileInfo,
            uint uFlags
            );

        [DllImport("User32.dll")]
        public static extern int DestroyIcon(IntPtr hIcon);

        [DllImport("shell32.dll", EntryPoint = "FindExecutable")]
        public static extern long FindExecutable(string lpFile, string lpDirectory, StringBuilder lpResult);
    }
}